@@ -1,4 +1,4 @@
-# Copyright 2001-2006, Paul Johnson (pjcj@cpan.org)
+# Copyright 2001-2009, Paul Johnson (pjcj@cpan.org)
 
 # This software is free.  It is licensed under the same terms as Perl itself.
 
@@ -10,13 +10,13 @@ package Devel::Cover;
 use strict;
 use warnings;
 
-our $VERSION = "0.59";
+our $VERSION = "0.65";
 
 use DynaLoader ();
 our @ISA = "DynaLoader";
 
-use Devel::Cover::DB  0.59;
-use Devel::Cover::Inc 0.59;
+use Devel::Cover::DB  0.65;
+use Devel::Cover::Inc 0.65;
 
 use B qw( class ppname main_cv main_start main_root walksymtable OPf_KIDS );
 use B::Debug;
@@ -66,6 +66,7 @@ my @Cvs;                                 # All the Cvs we want to cover.
 my @Subs;                                # All the subs we want to cover.
 my $Cv;                                  # Cv we are looking in.
 my $Sub_name;                            # Name of the sub we are looking in.
+my $Sub_count;                           # Count for multiple subs on same line.
 
 my $Coverage;                            # Raw coverage data.
 my $Structure;                           # Structure of the files.
@@ -90,6 +91,7 @@ BEGIN
     ($File, $Line, $Collect) = ("", 0, 1);
     $Silent = ($ENV{HARNESS_PERL_SWITCHES} || "") =~ /Devel::Cover/ ||
               ($ENV{PERL5OPT}              || "") =~ /Devel::Cover/;
+    *OUT = $ENV{DEVEL_COVER_DEBUG} ? *STDERR : *STDOUT;
 }
 
 if (0 && $Config{useithreads})
@@ -100,15 +102,15 @@ if (0 && $Config{useithreads})
 
     my $original_join;
     BEGIN { $original_join = \&threads::join }
-    # print "original_join: $original_join\n";
+    # print STDERR "original_join: $original_join\n";
 
-    # $original_join = sub { print "j\n" };
+    # $original_join = sub { print STDERR "j\n" };
 
     # sub threads::join
     *threads::join = sub
     {
-        # print "threads::join- ", \&threads::join, "\n";
-        # print "original_join- $original_join\n";
+        # print STDERR "threads::join- ", \&threads::join, "\n";
+        # print STDERR "original_join- $original_join\n";
         my $self = shift;
         print STDERR "(joining thread ", $self->tid, ")\n";
         my @ret = $original_join->($self, @_);
@@ -126,7 +128,7 @@ if (0 && $Config{useithreads})
         $original_destroy->($self, @_);
     };
 
-    # print "threads::join: ", \&threads::join, "\n";
+    # print STDERR "threads::join: ", \&threads::join, "\n";
 
     my $new = \&threads::new;
     *threads::new = *threads::create = sub
@@ -138,12 +140,12 @@ if (0 && $Config{useithreads})
         $new->($class,
                sub
                {
-                   print "Starting thread\n";
+                   print STDERR "Starting thread\n";
                    set_coverage(keys %Coverage);
                    my $ret = [ $sub->(@_) ];
-                   print "Ending thread\n";
+                   print STDERR "Ending thread\n";
                    report() if $Initialised;
-                   print "Ended thread\n";
+                   print STDERR "Ended thread\n";
                    $wantarray ? @{$ret} : $ret->[0];
                },
                @_
@@ -153,7 +155,8 @@ if (0 && $Config{useithreads})
 
 BEGIN { @Inc = @Devel::Cover::Inc::Inc; @Ignore = ("/Devel/Cover[./]") }
 # BEGIN { $^P = 0x004 | 0x010 | 0x100 | 0x200 }
-BEGIN { $^P = 0x004 | 0x100 | 0x200 }
+# BEGIN { $^P = 0x004 | 0x100 | 0x200 }
+BEGIN { $^P = 0x004 | 0x100 }
 
 {
     sub check
@@ -172,7 +175,7 @@ BEGIN { $^P = 0x004 | 0x100 | 0x200 }
         {
             delete $Coverage{pod};  # Pod::Coverage unavailable
             $nopod = <<EOM;
-    Pod coverage is unvailable.  Please install Pod::Coverage from CPAN.
+    Pod coverage is unavailable.  Please install Pod::Coverage from CPAN.
 EOM
         }
 
@@ -180,7 +183,7 @@ EOM
         @coverage = get_coverage();
         my $last = pop @coverage || "";
 
-        print STDOUT __PACKAGE__, " $VERSION: Collecting coverage data for ",
+        print OUT __PACKAGE__, " $VERSION: Collecting coverage data for ",
               join(", ", @coverage),
               @coverage ? " and " : "",
               "$last.\n",
@@ -206,14 +209,14 @@ EOM
     my $run_end = 0;
     sub first_end
     {
-        # print "**** END 1 - $run_end\n";
+        # print STDERR "**** END 1 - $run_end\n";
         set_last_end() unless $run_end++
     }
 
     my $run_init = 0;
     sub first_init
     {
-        # print "**** INIT 1 - $run_init\n";
+        # print STDERR "**** INIT 1 - $run_init\n";
         collect_inits() unless $run_init++
     }
 }
@@ -253,7 +256,7 @@ sub import
     my $class = shift;
 
     my @o = (@_, split ",", $ENV{DEVEL_COVER_OPTIONS} || "");
-    # print __PACKAGE__, ": Parsing options from [@_]\n";
+    # print STDERR __PACKAGE__, ": Parsing options from [@_]\n";
 
     my $blib = -d "blib";
     @Inc     = () if "@o" =~ /-inc /;
@@ -313,6 +316,7 @@ sub import
     @Select_re = map qr/$_/,                           @Select;
     @Ignore_re = map qr/$_/,                           @Ignore;
     @Inc_re    = map $ci ? qr/^\Q$_\//i : qr/^\Q$_\//, @Inc;
+    %Files     = ();  # start gathering file information from scratch
 
     for my $c (Devel::Cover::DB->new->criteria)
     {
@@ -405,8 +409,9 @@ sub normalised_file
 
     my $f = $file;
     $file =~ s/ \(autosplit into .*\)$//;
-    # print "file is <$file>\n";
-    # use Data::Dumper; print "file is <$file>\ncoverage is ", Dumper coverage(0);
+    # print STDERR "file is <$file>\n";
+    # use Data::Dumper;
+    # print STDERR "file is <$file>\ncoverage: ", Dumper coverage(0);
     if (exists coverage(0)->{module} && exists coverage(0)->{module}{$file} &&
         !File::Spec->file_name_is_absolute($file))
     {
@@ -424,20 +429,19 @@ sub normalised_file
         else
         {
             # print STDERR "getting abs_path <$file> ";
-            my $abs = abs_path($file);
+            my $abs;
+            $abs = abs_path($file) unless -l $file;  # leave symbolic links
             # print STDERR "giving <$file> ";
             $file = $abs if defined $abs;
         }
         # print STDERR "finally <$file> <$Dir>\n";
     }
-    if ($^O eq "MSWin32")
-    {
-        $file =~ s|\\|/|g;
-    }
+    $file =~ s|\\|/|g if $^O eq "MSWin32";
     $file =~ s|^$Dir/||;
-    $File_cache{$f} = $file;
 
-    # warn "File: $file => $File\n";
+    # print STDERR "File: $f => $file\n";
+
+    $File_cache{$f} = $file
 }
 
 }
@@ -484,10 +488,12 @@ sub use_file
     for (@Ignore_re) { return $Files{$file} = 0 if $f =~ $_ }
     for (@Inc_re)    { return $Files{$file} = 0 if $f =~ $_ }
 
-    # system "pwd; ls -l $file";
+    # system "pwd; ls -l '$file'";
     $Files{$file} = -e $file ? 1 : 0;
-    warn __PACKAGE__ . qq(: Can't find file "$file": ignored.\n)
-        unless $Files{$file} || $Silent || $file =~ /\(eval \d+\)/;
+    warn __PACKAGE__ . qq(: Can't find file "$file" (@_): ignored.\n)
+        unless $Files{$file} || $Silent || $file =~ /\(eval \d+\)/ ||
+               $file eq "../../lib/Storable.pm" ||
+               $file eq "../../lib/POSIX.pm";
 
     $Files{$file}
 }
@@ -503,7 +509,7 @@ sub check_file
 
     my $file = $op->file;
     my $use  = use_file($file);
-    # printf "%6s $file\n", $use ? "use" : "ignore";
+    # printf STDERR "%6s $file\n", $use ? "use" : "ignore";
 
     $use
 }
@@ -513,7 +519,7 @@ sub B::GV::find_cv
     my $cv = $_[0]->CV;
     return unless $$cv;
 
-    # print "find_cv $$cv\n" if check_file($cv);
+    # print STDERR "find_cv $$cv\n" if check_file($cv);
     push @Cvs, $cv if check_file($cv);
     push @Cvs, grep check_file($_), $cv->PADLIST->ARRAY->ARRAY
         if $cv->can("PADLIST") &&
@@ -530,7 +536,7 @@ sub sub_info
     {
         return unless $cv->GV->can("SAFENAME");
         $name = $cv->GV->SAFENAME;
-        # print "--[$name]--\n";
+        # print STDERR "--[$name]--\n";
         $name =~ s/(__ANON__)\[.+:\d+\]/$1/ if defined $name;
     }
     my $root = $cv->ROOT;
@@ -553,7 +559,7 @@ sub sub_info
 
 sub check_files
 {
-    # print "Checking files\n";
+    # print STDERR "Checking files\n";
 
     @Cvs = grep check_file($_), B::main_cv->PADLIST->ARRAY->ARRAY;
 
@@ -572,7 +578,7 @@ sub check_files
             local ($Line, $File);
             get_location($start);
             $line = $Line;
-            # print "$name - $File:$Line\n";
+            # print STDERR "$name - $File:$Line\n";
         }
         ($line, $name)
     };
@@ -593,6 +599,8 @@ sub check_files
 
 sub report
 {
+    local @SIG{qw(__DIE__ __WARN__)};
+
     $Run{finish} = get_elapsed();
 
     die "Devel::Cover::import() not run: " .
@@ -609,7 +617,8 @@ sub report
     $Structure      = Devel::Cover::DB::Structure->new(base => $DB);
     $Structure->read_all;
     $Structure->add_criteria(@collected);
-    # use Data::Dumper; print STDERR "Start structure", Dumper $Structure;
+    # use Data::Dumper; $Data::Dumper::Indent = 1;
+    # use Data::Dumper; print STDERR "Start structure: ", Dumper $Structure;
 
     # print STDERR "Processing cover data\n@Inc\n";
     $Coverage = coverage(1) || die "No coverage data available.\n";
@@ -631,17 +640,17 @@ sub report
         get_cover($_)
             for get_ends()->isa("B::AV") ? get_ends()->ARRAY : ();
     }
-    # print "--- @Cvs\n";
+    # print STDERR "--- @Cvs\n";
     get_cover($_) for @Cvs;
 
     my %files;
     $files{$_}++ for keys %{$Run{count}}, keys %{$Run{vec}};
     for my $file (sort keys %files)
     {
-        # print "looking at $file\n";
+        # print STDERR "looking at $file\n";
         unless (use_file($file))
         {
-            # print "deleting $file\n";
+            # print STDERR "deleting $file\n";
             delete $Run{count}->{$file};
             delete $Run{vec}  ->{$file};
             $Structure->delete_file($file);
@@ -658,7 +667,7 @@ sub report
         $Structure->store_counts($file);
     }
 
-    # use Data::Dumper; print STDERR "End structure", Dumper $Structure;
+    # use Data::Dumper; print STDERR "End structure: ", Dumper $Structure;
 
     my $run = time . ".$$." . sprintf "%05d", rand 2 ** 16;
     my $cover = Devel::Cover::DB->new
@@ -678,7 +687,7 @@ sub report
 
     $cover->{db} = $DB;
 
-    print STDOUT __PACKAGE__, ": Writing coverage database to $DB\n"
+    print OUT __PACKAGE__, ": Writing coverage database to $DB\n"
         unless $Silent;
     $cover->write;
     $cover->print_summary if $Summary && !$Silent;
@@ -691,7 +700,7 @@ sub add_subroutine_cover
     get_location($op);
     return unless $File;
 
-    # print "Subroutine $Sub_name $Line:$File: ", $op->name, "\n";
+    # print STDERR "Subroutine $Sub_name $File:$Line: ", $op->name, "\n";
 
     my $key = get_key($op);
     my $val = $Coverage->{statement}{$key} || 0;
@@ -744,7 +753,7 @@ sub add_branch_cover
     my $c   = $Coverage->{condition}{$key};
 
     no warnings "uninitialized";
-    # print STDERR "*** add_branch_cover $File:$Line [$type][$c]\n";
+    # warn "add_branch_cover $File:$Line [$type][@{[join ', ', @$c]}]\n";
 
     if ($type eq "and" ||
         $type eq "or"  ||
@@ -776,6 +785,8 @@ sub add_branch_cover
         my $vec = $Run{vec}{$File}{branch};
         vec($vec->{vec}, $vec->{size}++, 1) = $_ ||= 0 ? 1 : 0 for @$c;
     }
+
+    # warn "branch $type %x [@$c] => [@{$ccount->{branch}[$n]}]\n", $$op;
 }
 
 my %condition_locations;
@@ -813,7 +824,8 @@ sub add_condition_cover
         $name = $r->first->name if $name eq "sassign";
         # TODO - exec?  any others?
         # print STDERR "Name [$name]\n";
-        if ($c->[5] || $name =~ /^const|s?refgen|gelem|die|undef|bless$/)
+        if ($c->[5] || $name =~
+            /^const|s?refgen|gelem|die|undef|bless|anon(?:list|hash)|scalar$/)
         {
             $c = [ $c->[3], $c->[1] + $c->[2] ];
             $count = 2;
@@ -887,7 +899,7 @@ sub deparse
 
         my $name = $op->can("name") ? $op->name : "Unknown";
 
-        # print "$class:$name at $File:$Line\n";
+        # print STDERR "$class:$name at $File:$Line\n";
 
         {
             # Collect everything under here.
@@ -895,7 +907,7 @@ sub deparse
             $deparse = eval { $Original{deparse}->($self, @_) };
             $deparse =~ s/^\010+//mg if defined $deparse;
             $deparse = "Deparse error: $@" if $@;
-            # print "Collect Deparse $op $$op => <$deparse>\n";
+            # print STDERR "Collect Deparse $op $$op => <$deparse>\n";
         }
 
         # Get the coverage on this op.
@@ -941,6 +953,12 @@ sub deparse
                     my $newop   = $false->first;
                     my $newcond = $newop->first;
                     my $newtrue = $newcond->sibling;
+                    if ($newcond->name eq "lineseq")
+                    {
+                        # lineseq to ensure correct line numbers in elsif()
+                        # Bug #37302 fixed by change #33710.
+                        $newcond = $newcond->first->sibling;
+                    }
                     # last in chain is OP_AND => no else
                     $false      = $newtrue->sibling;
                     { local $Collect; $newcond = $self->deparse($newcond, 1) }
@@ -956,7 +974,7 @@ sub deparse
         $deparse = eval { $Original{deparse}->($self, @_) };
         $deparse =~ s/^\010+//mg if defined $deparse;
         $deparse = "Deparse error: $@" if $@;
-        # print "Deparse => <$deparse>\n";
+        # print STDERR "Deparse => <$deparse>\n";
     }
 
     $deparse
@@ -1041,11 +1059,11 @@ sub get_cover
 
     ($Sub_name, my $start) = sub_info($cv);
 
-    # print "get_cover: <$Sub_name>\n";
+    # print STDERR "get_cover: <$Sub_name>\n";
     return unless defined $Sub_name;  # Only happens within Safe.pm, AFAIK.
 
     get_location($start) if $start;
-    # print "[[$File:$Line]]\n";
+    # print STDERR "[[$File:$Line]]\n";
     # return unless length $File;
     return if length $File && !use_file($File);
 
@@ -1058,10 +1076,12 @@ sub get_cover
             $Sub_name eq "__ANON__" && $Structure->get_sub_name eq "__ANON__")
         {
             # Merge instances of anonymous subs into one.
+            # TODO - multiple anonymous subs on the same line.
         }
         else
         {
-            $Structure->set_subroutine($Sub_name, $File, $Line);
+            my $count = $Sub_count->{$File}{$Line}{$Sub_name}++;
+            $Structure->set_subroutine($Sub_name, $File, $Line, $count);
             add_subroutine_cover($start)
                 if $Coverage{subroutine} || $Coverage{pod};  # pod requires subs
         }
@@ -1097,7 +1117,7 @@ sub get_cover
                 }
             }
             $Pod = "Pod::Coverage" if delete $opts{nocp};
-            # use Data::Dumper; print "$Pod, ", Dumper \%opts;
+            # use Data::Dumper; print STDERR "$Pod, ", Dumper \%opts;
             if ($Pod{$file} ||= $Pod->new(package => $pkg, %opts))
             {
                 my $covered;
@@ -1128,9 +1148,9 @@ sub get_cover
     # my $dd = @_ && ref $_[0]
                  # ? $deparse->deparse($_[0], 0)
                  # : $deparse->deparse_sub($cv, 0);
-    # print "get_cover: <$Sub_name>\n";
-    # print "[[$File:$Line]]\n";
-    # print "<$dd>\n";
+    # print STDERR "get_cover: <$Sub_name>\n";
+    # print STDERR "[[$File:$Line]]\n";
+    # print STDERR "<$dd>\n";
 
     no warnings "redefine";
     local *B::Deparse::deparse     = \&deparse;
@@ -1140,7 +1160,7 @@ sub get_cover
     my $de = @_ && ref $_[0]
                  ? $deparse->deparse($_[0], 0)
                  : $deparse->deparse_sub($cv, 0);
-    # print "<$de>\n";
+    # print STDERR "<$de>\n";
     $de
 }
 
@@ -1156,18 +1176,18 @@ Devel::Cover - Code coverage metrics for Perl
 
 =head1 SYNOPSIS
 
- perl -MDevel::Cover yourprog args
- cover
+To get coverage for an uninstalled module:
 
- perl -MDevel::Cover=-db,cover_db,-coverage,statement,time yourprog args
+ cover -test
 
-To test an uninstalled module:
+or
 
  cover -delete
  HARNESS_PERL_SWITCHES=-MDevel::Cover make test
  cover
 
-To test an uninstalled module which uses Module::Build (0.26 or later):
+To get coverage for an uninstalled module which uses Module::Build (0.26 or
+later):
 
  ./Build testcover
 
@@ -1175,20 +1195,38 @@ If the module does not use the t/*.t framework:
 
  PERL5OPT=-MDevel::Cover make test
 
+If you want to get coverage for a program:
+
+ perl -MDevel::Cover yourprog args
+ cover
+
+ perl -MDevel::Cover=-db,cover_db,-coverage,statement,time yourprog args
+
 =head1 DESCRIPTION
 
 This module provides code coverage metrics for Perl. Code coverage
-metrics describe how throroughly tests exercise code. By using
-Devel::Cover you can find areas of code not exercised by your tests
-and find out which tests to create to increase coverage. Code coverage
+metrics describe how thoroughly tests exercise code. By using
+Devel::Cover you can discover areas of code not exercised by your tests
+and determine which tests to create to increase coverage. Code coverage
 can be considered as an indirect measure of quality.
 
-If you can't guess by the version number this is an alpha release.
+I consider this software to have an alpha status.  By that I mean that I
+reserve the right to alter the interface in a backwards incompatible manner
+without incrementing the major version number.  I specifically do not mean
+that this software is full of bugs or missing key features.  Although I'm
+making no guarantees on that front either.  In short, if you are looking for
+code coverage software for Perl, you have probably come to the end of your
+search.  For more of my opinions on this subject, see
+http://pjcj.sytes.net/notes/2007/03/14#alpha
 
 Code coverage data are collected using a pluggable runops function which
 counts how many times each op is executed.  These data are then mapped
 back to reality using the B compiler modules.  There is also a statement
 profiling facility which needs a better backend to be really useful.
+This release also includes an experimental mode which replaces ops
+instead of using a pluggable runops function.  This provides a nice
+speed increase, but needs better testing before it becomes the default.
+You probably don't care about any of this.
 
 The F<cover> program can be used to generate coverage reports.
 
@@ -1298,7 +1336,7 @@ The inc directories are initially populated with the contents of the
 directories using -inc, or add to them using +inc.
 
 Although these options take regular expressions, you should not enclose the RE
-within //.
+within // or any other quoting characters.
 
 =head1 ENVIRONMENT
 
@@ -1396,17 +1434,17 @@ CV.  Hints, tips or patches to resolve this will be gladly accepted.
 
 =head1 BUGS
 
-Did I mention that this is alpha code?
+Almost certainly.
 
 See the BUGS file.  And the TODO file.
 
 =head1 VERSION
 
-Version 0.59 - 23rd August 2006
+Version 0.65 - 8th August 2009
 
 =head1 LICENCE
 
-Copyright 2001-2006, Paul Johnson (pjcj@cpan.org)
+Copyright 2001-2009, Paul Johnson (pjcj@cpan.org)
 
 This software is free.  It is licensed under the same terms as Perl itself.
 
