@@ -4,10 +4,10 @@ use strict;
 use warnings;
 use CGI;
 use Getopt::Long;
-use Devel::Cover::DB 0.55;
-use Devel::Cover::Truth_Table 0.55;
+use Devel::Cover::DB 0.65;
+use Devel::Cover::Truth_Table 0.65;
 
-our $VERSION = "0.55";
+our $VERSION = "0.65";
 
 #-------------------------------------------------------------------------------
 # Subroutine : get_coverage_for_line
@@ -256,23 +256,23 @@ sub print_html_header {
     my $title = shift;
 
     print $fh <<"END_HTML";
+<!DOCTYPE html
+     PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
+     "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
+<html xmlns="http://www.w3.org/1999/xhtml">
 <!--
-This file was generated by Devel::Cover Version 0.55
-Devel::Cover is copyright 2001-2004, Paul Johnson (pjcj\@cpan.org)
+This file was generated by Devel::Cover Version 0.65
+Devel::Cover is copyright 2001-2006, Paul Johnson (pjcj\@cpan.org)
 Devel::Cover is free. It is licensed under the same terms as Perl itself.
 The latest version of Devel::Cover should be available from my homepage:
 http://www.pjcj.net
 -->
-
-<?xml version="1.0" encoding="utf-8"?>
-<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML Basic 1.0//EN"
-    "http://www.w3.org/TR/xhtml-basic/xhtml-basic10.dtd">
-<html xmlns="http://www.w3.org/1999/xhtml">
 <head>
-<meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1"></meta>
-<meta http-equiv="Content-Language" content="en-us"></meta>
-<link rel="stylesheet" type="text/css" href="cover.css"></link>
-<title>$title</title>
+    <meta http-equiv="Content-Type" content="text/html; charset=utf-8"></meta>
+    <meta http-equiv="Content-Language" content="en-us"></meta>
+    <link rel="stylesheet" type="text/css" href="cover.css"></link>
+    <link rel="stylesheet" type="text/css" href="cover.css"></link>
+    <title>$title</title>
 </head>
 END_HTML
 
@@ -288,7 +288,7 @@ sub print_summary {
     my $fh       = shift;
     my $title    = shift;
     my $file     = shift;
-    my $percent  = sprintf("%.1f", shift @_);
+    my $percent  = sprintf("%.1f", shift @_ || 0);
     my $error    = shift;
     my $db       = shift;
     my $class    = pclass($percent, $error);
@@ -522,6 +522,7 @@ sub print_file_report {
                         $link = get_link($fin, $c, $.);
                     }
     
+                    no warnings "uninitialized";  # TODO - hack, get rid of this
                     my $text = '<div';
                     $text .= $m->{class} ? qq' class="$m->{class}"' : '';
                     $text .= $m->{title} ? qq' title="$m->{title}"' : '';
@@ -673,7 +674,7 @@ sub get_options
     $opt->{option}{pod}          = 1;
     $opt->{option}{outputfile}   = "coverage.html";
     $opt->{option}{summarytitle} = "Coverage Summary";
-    die "Bad option" unless
+    die "Invalid command line options" unless
         GetOptions($opt->{option},
                    qw(
                        data!
@@ -733,7 +734,7 @@ Carman (mjcarman@mchsi.com).
 
 =head1 OPTIONS
 
-Options are specified by adding the appropraite flags to the C<cover> program.
+Options are specified by adding the appropriate flags to the C<cover> program.
 This report format supports the following:
 
 =over 4
@@ -775,11 +776,11 @@ Devel::Cover
 
 =head1 VERSION
 
-Version 0.55 - 22nd September 2005
+Version 0.65 - 8th August 2009
 
 =head1 LICENCE
 
-Copyright 2001-2005, Paul Johnson (pjcj@cpan.org)
+Copyright 2001-2009, Paul Johnson (pjcj@cpan.org)
 
 This software is free. It is licensed under the same terms as Perl itself.
 
@@ -819,7 +820,8 @@ a:visited {
 }
 
 table {
-	border-spacing: 1px;
+    border-collapse: collapse;
+	border-spacing: 0px;
 }
 tr {
 	text-align : center;
@@ -828,8 +830,7 @@ tr {
 th,.h {
 	background-color: #cccccc;
 	border: solid 1px #333333;
-	padding-left:  0.2em;
-	padding-right: 0.2em;
+    padding: 0em 0.2em;
 }
 td {
 	border: solid 1px #cccccc;
@@ -849,6 +850,7 @@ pre,.s {
  *   c2  : coverage >= 90%
  *   c3  : path covered or coverage = 100%
  */
+.c0, .c1, .c2, .c3 { text-align: right; }
 .c0 {
 	background-color: #ff9999;
 	border: solid 1px #cc0000;
